/*************************************************************************/
/*************************************************************************/
/* H. Timucin OZDEMIR                                                    */
/* Date : April, 1997                                                    */
/* File Name : FacilitySchedule.cc                                       */
/*************************************************************************/
/*************************************************************************/

#include <stdio.h>
#include <iostream.h>
#include "FacilitySchedule.h"
#include "deffunc.h"
#include "Operation.h"

FacilitySchedule::~FacilitySchedule()
{  
   //cout << " de allocate FacilitySchedule " << pairIndex << endl;
   if((pairIndex>0)&&(PairList!=NULL)) delete [] PairList;

}// end of destructor

FacilitySchedule::FacilitySchedule()
{  
   NumOfPairs = -1; 
   PairList   = NULL;
   pairIndex  = 0;
   AvailableTime = 0;
   debug = 0;

}// end of constructor

FacilitySchedule::FacilitySchedule(const FacilitySchedule& RHS)
{ 
   NumOfPairs = RHS.NumOfPairs; 
   pairIndex  = RHS.pairIndex;
   AvailableTime = RHS.AvailableTime;
   debug = RHS.debug;
   PairList   = new Operation[pairIndex];
   for(int i=0;i<pairIndex;i++) PairList[i] = RHS.PairList[i];

}// end of COPY constructor

void FacilitySchedule::debugOn() { debug=1; }

void FacilitySchedule::debugOff() { debug=0; }

FacilitySchedule::FacilitySchedule(int _PairNumber)
{ 
   NumOfPairs = _PairNumber; 
   //PairList   = new Operation[_PairNumber];
   PairList   = NULL;
   pairIndex  = 0;
   AvailableTime = 0;

}// end of constructor

/*
//  returns the number of operation that are scheduled to this machine 
//  so far.
*/

FacilitySchedule::getNumberOfOperations() const { return pairIndex; }

/*
// Adds the new operation with Time time stamp if possible. If this time
// is not appropriate for the machine's schedule, then machine's time
// value will be the starting time of the operation.
*/

void FacilitySchedule::addOperation(Operation newPair, int Time)
{  int i ;

   if(debug) cout << " I am in  the addOperation " << endl;

   Operation *temp = new Operation[pairIndex+1];
   for(i=0;i<pairIndex;i++) temp[i]=PairList[i];

   temp[i] = newPair;

   if(pairIndex>0) delete [] PairList;

   PairList = &(temp[0]);

   PairList[i].setStartTime(maximum(AvailableTime,Time));
   AvailableTime=maximum(AvailableTime,Time)+1+PairList[i].getElapsedTime();
   pairIndex++;
   if(debug) cout << " I am out of addOperation " << endl;

}// end of addOperation()

/*
//  Prints ou the current schedule to the standart output.
*/

void FacilitySchedule::print()
{  int i;
   for(i=0;i<pairIndex-1;i++) cout << i << PairList[i] << " , " ;
   if(i<pairIndex) cout << i << PairList[i] << endl;
}// end of print()

/*
//  returns the pointer of the operation asked with the index.
//  If the index is out of range, then it will return the NULL.
*/

Operation* FacilitySchedule::getOperationWithIndex(int indx)
{ return(indx>=0&&indx<pairIndex?&PairList[indx]:NULL); 
}// end of getOperationWithIndex()

/*
//  returns the first available empty time slot
*/

FacilitySchedule::getFirstEmptySlot()
{ return AvailableTime ; 
}// end of getFirstEmptySlot()

/*
//  Swaps the order of two operations on the schedule
*/

void FacilitySchedule::swapJobs(int FJobIndx, int SJobIndx)
{  
   if((FJobIndx>=0&&FJobIndx<pairIndex)&&(SJobIndx>=0&&SJobIndx<pairIndex))
   {  Operation knew = PairList[FJobIndx];
      PairList[FJobIndx] = PairList[SJobIndx];
      PairList[SJobIndx] = knew;
   }// end of if

}// end of swapJobs()

/*
//  Swaps the order of operations between two operations on the schedule
*/

void FacilitySchedule::swapJobsOrder(int FJobIndx, int SJobIndx)
{  
   if((FJobIndx>=0&&FJobIndx<pairIndex)&&(SJobIndx>=0&&SJobIndx<pairIndex))
   {  int lim = SJobIndx-FJobIndx+1; 
      Operation *knew = new Operation[lim];

      for(unsigned i=0;i<lim;i++) knew[i] = PairList[FJobIndx+i];
      for(i=0;i<lim;i++) PairList[SJobIndx-i] = knew[i];
      delete [] knew; 

   }// end of if

}// end of swapJobsOrder()

/*
//  Returns the finishing time of the given operation if it is already
//  scheduled to the machine. If it is not scheduled, then it returns 
//  0 value.
*/

FacilitySchedule::getTime(Operation Op)
{  int i=0, notFound=1;
   
   while(i<pairIndex&&notFound)
     if(PairList[i]==Op) notFound=0;
     else i++;

   return (notFound?0:PairList[i].getStartTime()+PairList[i].getElapsedTime());

}// end of getTime()

/*
//  Returns the start time of the given operation if it is already
//  scheduled. Otherwise, it will return a ZERO value.
*/

FacilitySchedule::getStartTime(Operation Op)
{  int i=0, notFound=1;
   
   while(i<pairIndex&&notFound)
     if(PairList[i]==Op) notFound=0;
     else i++;

   return (notFound?0:PairList[i].getStartTime());

}// end of getStartTime()

/*
//  Returns the start time of the given operation if it is already
//  scheduled. Otherwise, it will return a ZERO value.
*/

FacilitySchedule::getStartTimeOpWithIndx(int OpIndx)
{  
   return (OpIndx>=0 && OpIndx<pairIndex?0:PairList[OpIndx].getStartTime());

}// end of getStartTimeOpWithIndx()

/*
//  Returns the ending time of the given operation if it is already
//  scheduled. Otherwise, it will return a ZERO value.
*/

FacilitySchedule::getEndTime(Operation Op)
{  
   return (getTime(Op));

}// end of getEndTime()

/*
//  Returns the ending time of the given operation if it is already
//  scheduled. Otherwise, it will return a ZERO value.
*/

FacilitySchedule::getEndTimeOpWithIndx(int OpIndx)
{  
   return (OpIndx>=0 && OpIndx<pairIndex?
	   0:PairList[OpIndx].getStartTime()+PairList[OpIndx].getElapsedTime());

}// end of getStartTimeOpWithIndx()



ostream& operator<<(ostream &os, const FacilitySchedule & Schedule)
{  
   int i;

   for(i=0;i<Schedule.pairIndex-1;i++) 
       os << (i%3==0?"\n\t":" ") << i << Schedule.PairList[i] << " , ";

   if(i<Schedule.pairIndex) 
       os << (i%3==0?"\n\t":" ") << i << Schedule.PairList[i] ;

   return os;

}// end of operator <<

FacilitySchedule FacilitySchedule::operator =(const FacilitySchedule &RHSPair)
{  
   if(pairIndex) delete [] PairList;
   AvailableTime = RHSPair.AvailableTime;
   NumOfPairs    = RHSPair.NumOfPairs;
   pairIndex     = RHSPair.pairIndex;
   PairList      = new Operation[RHSPair.pairIndex];
   for(int i=0;i<RHSPair.pairIndex;i++)
        PairList[i]      =  RHSPair.PairList[i]; 

   return (*this);

}// end of operator =

FacilitySchedule::operator ==(const FacilitySchedule &RHSPair)
{  int ret=0, i;

   ret=(AvailableTime == RHSPair.AvailableTime &&
        NumOfPairs    == RHSPair.NumOfPairs &&
        pairIndex     == RHSPair.pairIndex );

   for(i=0;i<RHSPair.pairIndex && ret;i++)
        ret=(PairList[i]     ==  RHSPair.PairList[i]); 

   return ret;

}// end of operator ==

